@@ -23,6 +23,7 @@
 #include "config.h"
 #include "libopenbios/openbios.h"
 #include "libopenbios/bindings.h"
+#include "libopenbios/console.h"
 #include "drivers/pci.h"
 #include "arch/common/nvram.h"
 #include "drivers/drivers.h"
@@ -162,6 +163,8 @@ static const pci_arch_t known_arch[] = {
 };
 unsigned long isa_io_base;
 
+extern struct _console_ops mac_console_ops, prep_console_ops;
+
 void
 entry(void)
 {
@@ -184,6 +187,14 @@ entry(void)
 
     isa_io_base = arch->io_base;
 
+#ifdef CONFIG_DEBUG_CONSOLE
+    if (is_apple()) {
+        init_console(mac_console_ops);
+    } else {
+        init_console(prep_console_ops);
+    }
+#endif
+
     if (temp != 1) {
         printk("Incompatible configuration device version, freezing\n");
         for (;;) {
@@ -816,12 +827,17 @@ arch_of_init(void)
 #endif
 
     if (fw_cfg_read_i16(FW_CFG_NOGRAPHIC)) {
-        if (CONFIG_SERIAL_PORT) {
-           stdin_path = "scca";
-           stdout_path = "scca";
+        if (is_apple()) {
+            if (CONFIG_SERIAL_PORT) {
+                stdin_path = "scca";
+                stdout_path = "scca";
+            } else {
+                stdin_path = "sccb";
+                stdout_path = "sccb";
+            }
         } else {
-           stdin_path = "sccb";
-           stdout_path = "sccb";
+            stdin_path = "ttya";
+            stdout_path = "ttya";
         }
 
         /* Some bootloaders force the output to the screen device, so
@@ -836,8 +852,13 @@ arch_of_init(void)
         push_str("screen");
         fword("property");
     } else {
-        stdin_path = "adb-keyboard";
-        stdout_path = "screen";
+        if (is_apple()) {
+            stdin_path = "adb-keyboard";
+            stdout_path = "screen";
+        } else {
+            stdin_path = "keyboard";
+            stdout_path = "screen";
+        }
     }
 
     kvm_of_init();
